@@ -1,7 +1,7 @@
 /*
  * The main header file for SIP.
  *
- * Copyright (c) 2021 Riverbank Computing Limited <info@riverbankcomputing.com>
+ * Copyright (c) 2022 Riverbank Computing Limited <info@riverbankcomputing.com>
  *
  * This file is part of SIP.
  *
@@ -84,7 +84,7 @@
 /* Handle module flags. */
 
 #define MOD_HAS_DELAYED_DTORS   0x0001  /* It has a class with a delayed dtor. */
-#define MOD_IS_CONSOLIDATED     0x0002  /* It is a consolidated module. */
+#define MOD_IS_UNUSED           0x0002  /* This flag is unused. */
 #define MOD_IS_COMPOSITE        0x0004  /* It is a composite module. */
 #define MOD_IS_TRANSFORMED      0x0008  /* It's types have been transformed. */
 #define MOD_USE_ARG_NAMES       0x0010  /* Use real argument names. */
@@ -99,11 +99,8 @@
 
 #define hasDelayedDtors(m)  ((m)->modflags & MOD_HAS_DELAYED_DTORS)
 #define setHasDelayedDtors(m)   ((m)->modflags |= MOD_HAS_DELAYED_DTORS)
-#define isConsolidated(m)   ((m)->modflags & MOD_IS_CONSOLIDATED)
-#define setIsConsolidated(m)    ((m)->modflags |= MOD_IS_CONSOLIDATED)
 #define isComposite(m)      ((m)->modflags & MOD_IS_COMPOSITE)
 #define setIsComposite(m)   ((m)->modflags |= MOD_IS_COMPOSITE)
-#define isContainer(m)      ((m)->modflags & (MOD_IS_CONSOLIDATED | MOD_IS_COMPOSITE))
 #define setIsTransformed(m) ((m)->modflags |= MOD_IS_TRANSFORMED)
 #define isTransformed(m)    ((m)->modflags & MOD_IS_TRANSFORMED)
 #define setUseArgNames(m)   ((m)->modflags |= MOD_USE_ARG_NAMES)
@@ -307,11 +304,12 @@
 #define ENUM_NO_SCOPE       0x00000200  /* Omit the member scopes. */
 #define ENUM_NEEDS_ENUM     0x00000400  /* The module needs it. */
 #define ENUM_SCOPED         0x00000800  /* A C++0x11 scoped enum. */
-#define ENUM_MASK           0x00003000  /* The enum base type mask. */
+#define ENUM_MASK           0x00007000  /* The enum base type mask. */
 #define ENUM_ENUM           0x00000000  /* The base type is Enum. */
 #define ENUM_FLAG           0x00001000  /* The base type is Flag. */
 #define ENUM_INT_ENUM       0x00002000  /* The base type is IntEnum. */
 #define ENUM_INT_FLAG       0x00003000  /* The base type is IntFlag. */
+#define ENUM_UINT_ENUM      0x00004000  /* The base type is IntEnum with unsigned members. */
 
 #define isProtectedEnum(e)  ((e)->enumflags & SECT_IS_PROT)
 #define setIsProtectedEnum(e)   ((e)->enumflags |= SECT_IS_PROT)
@@ -335,18 +333,17 @@
 #define setIsEnumIntEnum(e) ((e)->enumflags |= ENUM_INT_ENUM)
 #define isEnumIntFlag(e)    (((e)->enumflags & ENUM_MASK) == ENUM_INT_FLAG)
 #define setIsEnumIntFlag(e) ((e)->enumflags |= ENUM_INT_FLAG)
+#define isEnumUIntEnum(e)   (((e)->enumflags & ENUM_MASK) == ENUM_UINT_ENUM)
+#define setIsEnumUIntEnum(e)    ((e)->enumflags |= ENUM_UINT_ENUM)
 
 
 /* Handle hierarchy flags. */
 
 #define HIER_BEING_SET      0x0001      /* The MRO is being set. */
-#define IN_A_DIAMOND        0x0002      /* The class is in a diamond. */
 
 #define hierBeingSet(m)     ((m)->mroflags & HIER_BEING_SET)
 #define setHierBeingSet(m)  ((m)->mroflags |= HIER_BEING_SET)
 #define resetHierBeingSet(m)    ((m)->mroflags &= ~HIER_BEING_SET)
-#define inADiamond(m)       ((m)->mroflags & IN_A_DIAMOND)
-#define setInADiamond(m)    ((m)->mroflags |= IN_A_DIAMOND)
 
 
 /* Handle overload flags.  These are combined with the section flags. */
@@ -796,13 +793,6 @@ typedef struct _qualDef {
 } qualDef;
 
 
-/* A platform. */
-typedef struct _platformDef {
-    struct _qualDef *qualifier;         /* The platform qualifier. */
-    struct _platformDef *next;          /* Next in the list. */
-} platformDef;
-
-
 /* A scoped name. */
 typedef struct _scopedNameDef {
     char *name;                         /* The name. */
@@ -1345,7 +1335,6 @@ typedef struct {
     virtErrorHandler *errorhandlers;    /* The list of virtual error handlers. */
     codeBlockList *exphdrcode;          /* Exported header code. */
     codeBlockList *exptypehintcode;     /* Exported type hint code. */
-    codeBlockList *docs;                /* Documentation. */
     classDef *qobject_cd;               /* QObject class, NULL if none. */
     int genc;                           /* Set if we are generating C code. */
     struct _stringList *plugins;        /* The list of plugins. */
@@ -1372,7 +1361,6 @@ extern unsigned sipVersion;             /* The version of SIP. */
 extern const char *sipVersionStr;       /* The version of SIP as a string. */
 extern char *sipName;                   /* The name of the sip module. */
 extern unsigned abiVersion;             /* The encoded ABI version number. */
-extern stringList *includeDirList;      /* The include directory list for SIP files. */
 
 
 void parse(sipSpec *, FILE *, char *, int, stringList **, stringList *,
@@ -1416,7 +1404,9 @@ scopedNameDef *copyScopedName(scopedNameDef *);
 void appendScopedName(scopedNameDef **,scopedNameDef *);
 scopedNameDef *text2scopePart(char *text);
 void freeScopedName(scopedNameDef *);
+char *scopedNameToString(scopedNameDef *name);
 void appendToClassList(classList **,classDef *);
+void appendCodeBlock(codeBlockList **headp, codeBlock *cb);
 void appendCodeBlockList(codeBlockList **headp, codeBlockList *cbl);
 void prcode(FILE *fp, const char *fmt, ...);
 void prCopying(FILE *fp, moduleDef *mod, const char *comment);
@@ -1432,9 +1422,11 @@ int isVoidReturnSlot(memberDef *md);
 int isNumberSlot(memberDef *md);
 int isInplaceNumberSlot(memberDef *md);
 int isRichCompareSlot(memberDef *md);
-mappedTypeDef *allocMappedType(sipSpec *pt, argDef *type);
+mappedTypeDef *allocMappedType(sipSpec *pt, argDef *type, int use_name);
 void appendString(stringList **headp, const char *s);
-void appendTypeStrings(scopedNameDef *ename, signatureDef *patt, signatureDef *src, signatureDef *known, scopedNameDef **names, scopedNameDef **values);
+void templateExpansions(signatureDef *patt, signatureDef *src,
+        signatureDef *declared_names, scopedNameDef **names,
+        scopedNameDef **values);
 codeBlockList *templateCode(sipSpec *pt, ifaceFileList **used,
         codeBlockList *ocbl, scopedNameDef *names, scopedNameDef *values);
 ifaceFileDef *findIfaceFile(sipSpec *pt, moduleDef *mod,
@@ -1448,7 +1440,6 @@ nameDef *cacheName(sipSpec *pt, const char *name);
 scopedNameDef *encodedTemplateName(templateDef *td);
 memberDef *findMethod(classDef *cd, const char *name);
 typeHintDef *newTypeHint(char *raw_hint);
-int isPyKeyword(const char *word);
 char *templateString(const char *src, scopedNameDef *names,
         scopedNameDef *values);
 void dsCtor(sipSpec *pt, classDef *cd, ctorDef *ct, FILE *fp);
@@ -1465,183 +1456,6 @@ void normaliseArgs(signatureDef *sd);
 void restoreArgs(signatureDef *sd);
 void initialiseLexer(void);
 int usedInCode(codeBlockList *cbl, const char *str);
-
-
-/* These are only here because bison publically references them. */
-
-/* Represent a set of option flags. */
-
-#define MAX_NR_FLAGS    5
-
-typedef enum {
-    bool_flag,
-    string_flag,
-    string_list_flag,
-    name_flag,
-    opt_name_flag,
-    dotted_name_flag,
-    integer_flag,
-    opt_integer_flag,
-} flagType;
-
-typedef struct {
-    const char *fname;                  /* The flag name. */
-    flagType ftype;                     /* The flag type. */
-    union {                             /* The flag value. */
-        char *sval;                     /* A string value. */
-        struct _stringList *slval;      /* A string list value. */
-        long ival;                      /* An integer value. */
-    } fvalue;
-} optFlag;
-
-typedef struct {
-    int nrFlags;                        /* The number of flags. */
-    optFlag flags[MAX_NR_FLAGS];        /* Each flag. */
-} optFlags;
-
-
-/* These represent the configuration of different directives. */
-
-/* %AutoPyName */
-typedef struct _autoPyNameCfg {
-    int token;
-    const char *remove_leading;
-} autoPyNameCfg;
-
-/* %CompositeModule */
-typedef struct _compModuleCfg {
-    int token;
-    const char *name;
-    docstringDef *docstring;
-} compModuleCfg;
-
-/* %DefaultDocstringFormat */
-typedef struct _defDocstringFmtCfg {
-    int token;
-    const char *name;
-} defDocstringFmtCfg;
-
-/* %DefaultDocstringSignature */
-typedef struct _defDocstringSigCfg {
-    int token;
-    const char *name;
-} defDocstringSigCfg;
-
-/* %DefaultEncoding */
-typedef struct _defEncodingCfg {
-    int token;
-    const char *name;
-} defEncodingCfg;
-
-/* %DefaultMetatype */
-typedef struct _defMetatypeCfg {
-    int token;
-    const char *name;
-} defMetatypeCfg;
-
-/* %DefaultSupertype */
-typedef struct _defSupertypeCfg {
-    int token;
-    const char *name;
-} defSupertypeCfg;
-
-/* %Docstring */
-typedef struct _docstringCfg {
-    int token;
-    Format format;
-    Signature signature;
-} docstringCfg;
-
-/* %Exception */
-typedef struct _exceptionCfg {
-    int token;
-    codeBlock *type_header_code;
-    codeBlock *raise_code;
-} exceptionCfg;
-
-/* %Extract */
-typedef struct _extractCfg {
-    int token;
-    const char *id;
-    int order;
-} extractCfg;
-
-/* %Feature */
-typedef struct _featureCfg {
-    int token;
-    const char *name;
-} featureCfg;
-
-/* %HiddenNamespace */
-typedef struct _hiddenNsCfg {
-    int token;
-    scopedNameDef *name;
-} hiddenNsCfg;
-
-/* %Import */
-typedef struct _importCfg {
-    int token;
-    const char *name;
-} importCfg;
-
-/* %Include */
-typedef struct _includeCfg {
-    int token;
-    const char *name;
-    int optional;
-} includeCfg;
-
-/* %License */
-typedef struct _licenseCfg {
-    int token;
-    const char *type;
-    const char *licensee;
-    const char *signature;
-    const char *timestamp;
-} licenseCfg;
-
-/* %Module and its sub-directives. */
-typedef struct _moduleCfg {
-    int token;
-    int c_module;
-    KwArgs kwargs;
-    const char *name;
-    int use_arg_names;
-    int py_ssize_t_clean;
-    int use_limited_api;
-    int all_raise_py_exc;
-    int call_super_init;
-    const char *def_error_handler;
-    docstringDef *docstring;
-} moduleCfg;
-
-/* %Plugin */
-typedef struct _pluginCfg {
-    int token;
-    const char *name;
-} pluginCfg;
-
-/* %Property */
-typedef struct _propertyCfg {
-    int token;
-    const char *get;
-    const char *name;
-    const char *set;
-    docstringDef *docstring;
-} propertyCfg;
-
-/* Variable sub-directives. */
-typedef struct _variableCfg {
-    int token;
-    codeBlock *access_code;
-    codeBlock *get_code;
-    codeBlock *set_code;
-} variableCfg;
-
-/* %VirtualErrorHandler */
-typedef struct _vehCfg {
-    int token;
-    const char *name;
-} vehCfg;
+int selectedQualifier(stringList *needed_qualifiers, qualDef *qd);
 
 #endif
